@@ -472,7 +472,7 @@ class DumpTests(unittest.TestCase):
 		self.assertEqual(buff1.getvalue(), '')
 		self.assertEqual(buff2.getvalue(), '')
 
-	def test_list_vspacing(self):
+	def test_vspacing_limits(self):
 		itm = self.yaml_var('''
 			builtIn: 1
 			datasource:
@@ -495,8 +495,42 @@ class DumpTests(unittest.TestCase):
 						uid: grafana
 					fieldConfig:''', raw=True)
 		for n in range(60): ys += '\n' + '  '*3 + f'field{n}: value-{n}'
-		ys = pyaml.dump(yaml.safe_load(ys))
+		ys = pyaml.dump(yaml.safe_load(ys), vspacing=dict(oneline_split=True))
 		self.assertEqual(self.empty_line_list(ys), list(range(4, 126, 2)))
+		ys = pyaml.dump(yaml.safe_load(ys))
+		self.assertEqual(self.empty_line_list(ys), [4])
+
+	def test_split_block_types(self):
+		data = self.yaml_var('''
+			test: # list items should be split
+				- key1: 1
+					key2: A
+				- key1: 1
+					key2: A
+				- key1: 1
+					key2: A
+			test2: # list items should be split
+				- key1:
+						key11: A
+						key12: B
+				- key2:
+						key21: A
+				- val
+			test3: # only top-level keys should be split
+				key1:
+					key11:
+					key12:
+				key2:
+					- key21: A
+					- key22: B
+					- key23: C
+				key3:
+					- key31:
+					- key32:
+					- key33:
+			''')
+		ys = pyaml.dump(data, vspacing=dict(split_lines=0))
+		self.assertEqual(self.empty_line_list(ys), [1, 4, 7, 10, 12, 16, 19, 21, 23, 27, 32])
 
 	def test_anchor_cutoff(self):
 		data = self.yaml_var('''
@@ -521,11 +555,16 @@ class DumpTests(unittest.TestCase):
 			self.assertLess(len(m[0]), 30)
 			self.assertEqual(len(re.findall(r'test\d', m[0])), 2)
 
-	def test_group_online_values(self):
+	def test_group_oneline_values(self):
 		data = self.yaml_var('''
 			similique-natus: 1
 			similique-commodi:
 				aperiam-libero: 2
+			'vel praesentium quo':
+				exercitationem debitis: porro beatae id
+				rerum commodi ipsum: nesciunt veritatis
+				amet quaerat:
+					assumenda: odio tenetur saepe
 			"111": digit-string
 			deserunt-est-2: asdasd
 			deserunt-est-1: |
@@ -536,13 +575,56 @@ class DumpTests(unittest.TestCase):
 			10: test1
 			200: test
 			30: test2''')
-		ys1 = pyaml.dump(data,
+		# Sort and oneline-group
+		ys1 = pyaml.dump( data,
 			sort_dicts=pyaml.PYAMLSort.oneline_group,
 			vspacing=dict(split_lines=0, split_count=0) )
-		self.assertEqual(self.empty_line_list(ys1), [8, 12])
+		self.assertEqual(self.empty_line_list(ys1), [8, 12, 15, 19])
+		# No oneline-group by default
 		ys2 = pyaml.dump(data, vspacing=dict(split_lines=0, split_count=0))
 		self.assertNotEqual(ys1, ys2)
-		self.assertEqual(self.empty_line_list(ys2), [1, 3, 5, 7, 9, 13, 15, 17, 19, 21])
+		self.assertEqual( self.empty_line_list(ys2),
+			[1, 4, 6, 9, 11, 13, 15, 17, 21, 23, 25, 27, 29] )
+		# No-sort oneline-group overrides oneline-split for conseq oneliners
+		ys3 = pyaml.dump(data, sort_keys=False, vspacing=dict(
+			split_lines=0, split_count=0, oneline_group=True, oneline_split=True ))
+		self.assertNotEqual(ys1, ys3)
+		self.assertNotEqual(ys2, ys3)
+		self.assertEqual(self.empty_line_list(ys3), [1, 4, 8, 11, 14, 18])
+
+	def test_repr_undefined(self):
+		class TestType1:
+			def __repr__(self): return large_yaml.decode()
+		class TestType2:
+			def __repr__(self): return '# should end up quoted' + ' '*100
+		unknown3 = type('TestType3', (object,), dict(a='b'))()
+		with open('/dev/null') as unknown4:
+			data = dict( known='val', unknown1=TestType1(),
+				unknown2=TestType2(), unknown3=unknown3, unknown4=unknown4 )
+		with self.assertRaises(yaml.YAMLError): ys = pyaml.dump(data)
+		with self.assertRaises(yaml.YAMLError): ys = pyaml.dump(unknown3)
+		self.assertIn(' ...[1/', pyaml.dump(data, repr_unknown=1))
+		ys = pyaml.dump(data, repr_unknown=True)
+		self.assertEqual(len(ys.splitlines()), 5)
+		self.assertIn('.test_dump.TestType3', ys)
+		self.assertIn('TextIOWrapper', ys)
+		ys = pyaml.dump(data, width=30, repr_unknown=30)
+		self.assertEqual(len(ys.splitlines()), 5)
+		self.assertIn(' ...[30/', ys)
+		self.assertNotIn('unknown2: # should end up quoted', ys)
+		self.assertIn('.test_dump.TestType3', ys)
+
+	def test_long_value_split(self):
+		data = dict(
+			sshkey=dict(__XML__PARAM__line=dict(__XML__value=f'ssh-rsa {"X"*372} user@host')),
+			vlan=[{'__XML__PARAM__vlan-id-create-delete': dict(__XML__value='1, 30, 50,'
+				' 110, 121-126, 999-1001, 1010, 1012, 1014, 1016, 1018, 1020-1022, 1060-1063,'
+				' 1080-1083, 1111-1113, 2000, 2201-2202, 2206, 2208, 2210, 2254, 3000, 3002,'
+				' 3006, 3008, 3021, 3100-3102, 3106, 3108, 3121, 3322, 3326, 3328, 3448-3452,'
+				' 3455-3458, 3950, 3955, 3960, 3965-3966')}] )
+		ys = pyaml.dump(data)
+		self.assertEqual(len(ys.splitlines()), 9)
+		self.assertEqual(yaml.safe_load(ys), data)
 
 
 if __name__ == '__main__':
